"""
Coil Calculator. Modified from NoteBook.py
Inspired by 'mini Ring Core Calculator' from  DL5SWB
"""

title = 'Bridge Calculator'

# Import Pmw from this directory tree.
import sys
#sys.path[:0] = ['../../..']

import os
#import math
from math import sin, cos, pi
import Tkinter
import Pmw

class BridgeCalc:
    def __init__(self, parent):

	group = Pmw.Group(parent, tag_text = 'Input')
	group.pack(fill = 'both', expand = 1, padx=10, pady=10)

	# Vin entry field
	l = Tkinter.Label(group.interior(),
		text='Vin:',
		anchor='e',
		width = 6
	)
	l.grid(row = 0, column = 0)
	self.Vin = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 10,
		errorbackground = None,
		#modifiedcommand = self.Vzchanged
	)
	self.Vin.grid(row=0, column=1, sticky='w')
	l = Tkinter.Label(group.interior(),
		text='V',
		anchor='w',
		width = 6
	)
	l.grid(row = 0, column = 2)

	# Vz entry field
	l = Tkinter.Label(group.interior(),
		text='Vz:',
		anchor='e',
		width = 6
	)
	l.grid(row = 1, column = 0)
	self.Vz = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real', 'min' : 0 },
		entry_width = 10,
		errorbackground = None,
		modifiedcommand = self.Vzchanged
	)
	self.Vz.grid(row=1, column=1, sticky='w')
	l = Tkinter.Label(group.interior(),
		text='V',
		anchor='w',
		width = 6
	)
	l.grid(row = 1, column = 2)

	# Phase entry field
	l = Tkinter.Label(group.interior(),
		text='Phase:',
		anchor='e',
		width = 6
	)
	l.grid(row = 2, column = 0)
	self.phase = Pmw.EntryField(group.interior(),
		labelpos = None,
		validate = { 'validator' : 'real'},
		entry_width = 10,
		errorbackground = None,
		modifiedcommand = self.Phchanged
	)
	self.phase.grid(row=2, column=1, sticky='w')
	l = Tkinter.Label(group.interior(),
		text='degrees',
		anchor='w',
		width = 8
	)
	l.grid(row = 2, column = 2)

	group = Pmw.Group(parent, tag_text = 'Output')
	group.pack(fill = 'both', expand = 1, padx=10, pady=10)

	self.Z = Tkinter.Label(group.interior(),
		text='',
		#anchor='w',
		width = 20
	)
	self.Z.grid(row = 0, column = 0, sticky='w')

    def Vzchanged(self):
	self.updateZ()

    def Phchanged(self):
	self.updateZ()

    def updateZ(self):
	Vin = self.Vin.getvalue()
	try:
		Vin = float(Vin)
	except:
		self.Z.configure(text = "")
		return
	Vz = self.Vz.getvalue()
	try:
		Vz = float(Vz)
	except:
		self.Z.configure(text = "")
		return
	phase = self.phase.getvalue()
	try:
		phase = float(phase)
	except:
		self.Z.configure(text = "")
		return
	# Convert to radians
	while phase < 0.0:
		phase += 360.0
	phase = phase / 180.0 * pi
	Vzr = Vz * cos(phase)
	Vzx = Vz * sin(phase)
	Vz = Vzr + Vzx * 1j

	# R1 = R2 = R3 = 50.0 ohm
	# Vz = Vin * Z / (Z + R3)
	# Z = R * Vz / (Vz - Vin)
	try:
		Z = 50.0 * Vz / (Vin - Vz)
	except:
		print "Z calculation failed"
	result = "Z = %.2f + %.2fj" % (Z.real, Z.imag)
	self.Z.configure(text = result)

class NotImplemented:
    def __init__(self, parent):
	w = Tkinter.Label(parent,
		text = 'Not implemented yet',
		relief = 'sunken'
	)
	w.pack(padx = 8, pady = 8)

class CoilCalc:
    def __init__(self, parent):
	# Create and pack the NoteBook.
	notebook = Pmw.NoteBook(parent)
	notebook.pack(fill = 'both', expand = 1, padx = 10, pady = 10)

	# Add IronPowder Page
	page = notebook.add('Bridge')
	p1 = BridgeCalc(page)

	# Add Ferrite page
	page = notebook.add('Tab2')
	p2 = NotImplemented(page)

	#notebook.setnaturalsize()

######################################################################

# Create demo in root window for testing.
if __name__ == '__main__':
    root = Tkinter.Tk()
    Pmw.initialise(root)
    root.title(title)

    print "pid =" , os.getpid()

    widget = BridgeCalc(root)
    exitButton = Tkinter.Button(root, text = 'Exit', command = root.destroy)
    exitButton.pack()
    root.mainloop()
